package controller;

import java.io.*;

import listener.CoordinateListener;
import model.*;
import ui.*;
import utils.*;
import zuozuomu.watermark.*;

public abstract class BambooController {
	protected BambooModel model;
	private BambooView view;
	private Coordinate coordinate;
	protected BambooParser parser = new BambooParser();

	public BambooController(final BambooModel model) {
		coordinate = model.getCoordinate();
		this.model = model;
	}

	protected void setView(BambooView view) {
		this.view = view;
	}

	public void cursorUp() {
		coordinate.up();
	}

	public void cursorDown() {
		coordinate.down();
	}

	public void cursorRight() {
		coordinate.right();
	}

	public void cursorLeft() {
		coordinate.left();
	}

	public void cursorShiftLeft() {
		coordinate.shiftLeft();
	}

	public void cursorShiftRight() {
		coordinate.shiftRight();
	}

	public void cursorShiftDown() {
		coordinate.shiftDown();
	}

	public void cursorShiftUp() {
		coordinate.shiftUp();
	}

	public void mouseDragged(int x, int y) {
		this.setPoint(x, y);
	}

	public void mousePressed(int x, int y) {
		this.setPoint0(x, y);
		this.setPoint(x, y);
	}

	public void cursorSpace() {
		if (coordinate.isRegion()) {
			deleteBamboo();
			coordinate.space();
		} else {
			cursorEnter();
		}
	}

	/** |n̋Lׂď */
	private void deleteBamboo() {
		model.getLayerList().deleteBamboo(coordinate.getXMin(),
				coordinate.getXMax(), coordinate.getYMin(),
				coordinate.getYMax());
	}

	public void cursorDelete() {
		model.deleteRegion();
		coordinate.delete();
	}

	public void cursorEnter() {
		model.setMap(Cell.MASU, coordinate.getXMin(), coordinate.getXMax(),
				coordinate.getYMin(), coordinate.getYMax());
		coordinate.enter();
	}

	public void setPoint0(int x, int y) {
		coordinate.setRegion(false);

		coordinate.setX0(x / view.getSizeOfCell() + 1);
		coordinate.setY0(y / view.getSizeOfCell() + 1);
	}

	public void setPoint(int locationX, int locationY) {
		int x = locationX / view.getSizeOfCell() + 1;
		int y = locationY / view.getSizeOfCell() + 1;

		coordinate.setX(x);
		coordinate.setY(y);

		int x0 = coordinate.getX0();
		int y0 = coordinate.getY0();

		if (x != x0 || y != y0) {
			coordinate.setRegion(true);
		}
	}

	public void newFrame(final int masu, final int size) {
		model.getLayerList().newMaps(masu, masu);
	}

	public void loadAtf(final File file) {
		AtfManager atf = new AtfManager(file);
		int width = atf.getWidth();
		int height = atf.getHeight();
		int[][] map = atf.getCells();

		newFrame(MathUtils.max(width, height, 20), 24);
		int ystart = (view.getCellsHeight() - height) / 2 + 1;
		int xstart = (view.getCellsWidth() - width) / 2 + 1;

		model.getLayerList().setMap(map, xstart, width, ystart, height);
	}

	public void loadDat(File file) {
		model.getLayerList().loadDat(file); // datt@C̃[h
	}

	public void decode(File file) {
		JikkenImage image = new JikkenImage(file);
		int[] header = image.extractIntArray(0, 0, 2);
		int[][] map = image.extractIntSquareArray(0, 1, header[1], header[0]);
		// }bvݒ
		newFrame(MathUtils.max(header[0], header[1], 20), 24);
		int ystart = (view.getCellsHeight() - header[1]) / 2 + 1;
		int xstart = (view.getCellsWidth() - header[0]) / 2 + 1;
		model.getLayerList().setMap(map, xstart, header[0], ystart, header[1]);
	}

	public void oldModel() {
		model.getLayerList().decreaseLayer();
		if (model.getLayerList().getLayer() == 0) {
			view.disableLeftButton();
		}
	}

	public void newModel() {
		model.getLayerList().increaseLayer();
		view.enableLeftButton();
	}

	/** V}̐ݒ */
	public void setData(int i) {
		model.setData(i);
		coordinate.notifyToCoordinateListeners();
	}

	/** |EV}̐ݒ */
	public void setBamboo(String string) {
		model.setBamboo(string);
		coordinate.notifyToCoordinateListeners();
	}

	/** ݂̈ʒuV}ɂ */
	public void makeIsland() {
		model.makeIsland();
		coordinate.cancelRegion();
		coordinate.notifyToCoordinateListeners();
	}

	public boolean isRegion() {
		return coordinate.isRegion();
	}

	public int getX() {
		return coordinate.getX();
	}

	public int getY() {
		return coordinate.getY();
	}

	public int getX0() {
		return coordinate.getX0();
	}

	public int getY0() {
		return coordinate.getY0();
	}

	public void setRegion(boolean b) {
		coordinate.setRegion(b);
	}

	public int getCellsWidth() {
		return model.getLayerList().getWidth() + 2;
	}

	public int getCellsHeight() {
		return model.getLayerList().getHeight() + 2;
	}

	public int getMap(int layer, int i, int j) {
		return model.getLayerList().getMap(layer, i, j);
	}

	public int getDataWidth() {
		return coordinate.getDataWidth();
	}

	public int getDataHeight() {
		return coordinate.getDataHeight();
	}

	public void addCoodinateListener(CoordinateListener coordinateListener) {
		coordinate.addCoordinateListener(coordinateListener);
	}

	public int getLayerSize() {
		return model.getLayerList().getLayerSize();
	}

	public int getLayer() {
		return model.getLayerList().getLayer();
	}

	public void deleteAllIslands() {
		model.getLayerList().deleteAllIslands();
	}

	public void setCross() {
		model.setCross();
		coordinate.notifyToCoordinateListeners();
	}

	public void url2() {
		String url = parser.toUrlFromModel(model);
		view.showURL(url);
	}

	public void url3() {
		String url = parser.toUrlFromModelB(model);
		view.showURL(url);
	}

	public int getSizeOfCell() {
		return view.getSizeOfCell();
	}

	public int getPanelWidth() {
		return view.getPanelWidth();
	}

	public int getPanelHeight() {
		return view.getPanelHeight();
	}

	public void reverse() {
		model.reverse();
	}

	public void rotate() {
		model.rotate();
	}

	public void unlock() {
		model.setState(BambooModel.State.UNLOCK);
		System.out.println("unlock");
	}

	public void halfLock() {
		model.setState(BambooModel.State.HALF_LOCK);
		System.out.println("half lock");
	}

	public void lock() {
		model.setState(BambooModel.State.LOCK);
		System.out.println("lock");
	}

}
