package model;

import java.io.*;

import ui.BambooView;

public class SaveController {
	private Coordinate coordinate;
	private int saveWidth;
	private int saveHeight;
	private int x;
	private int y;
	private LayerList layerList;
	private static final String urlfile = "url.txt";

	public SaveController(Coordinate coordinate, LayerList layerList) {
		this.coordinate = coordinate;
		this.layerList = layerList;
	}

	public void saveAtf(File file) {
		setSquare();
		PrintWriter writer = null;
		try {
			writer = new PrintWriter(new BufferedWriter(new FileWriter(file)));
		} catch (IOException e) {
			e.printStackTrace();
		}

		writer.print("size:");
		writer.println(saveWidth + "x" + saveHeight);

		layerList.saveAtf(writer, x, y, saveWidth, saveHeight);
		writer.close();
	}

	protected void setSquare() {
		coordinate.setRegion(false);
		saveWidth = 0;
		saveHeight = 0;

		for (int j = 1; j <= layerList.getHeight(); j++) {
			for (int i = 1; i <= layerList.getWidth(); i++) {
				if (layerList.getMap(0, i, j) != Cell.KARA) {
					x = i;
					y = j;
					int k = i;
					while ((layerList.getMap(0, k, j) != Cell.KARA)
							&& (layerList.getMap(0, k, j) != Cell.KABE)) {
						saveWidth++;
						k++;
					}
					k = j;
					while (layerList.getMap(0, i, k) != Cell.KARA
							&& layerList.getMap(0, i, k) != Cell.KABE) {
						saveHeight++;
						k++;
					}
					return;
				}
			}
		}

	}

	public void setSaveXY() {
		coordinate.setRegion(false);
		int x = coordinate.getX();
		int x0 = coordinate.getX0();
		int y = coordinate.getY();
		int y0 = coordinate.getY0();
		saveWidth = Math.abs(x - x0) + 1;
		saveHeight = Math.abs(y - y0) + 1;
		this.x = Math.min(x, x0);
		this.y = Math.min(y, y0);
	}

	public int getSaveWidth() {
		return saveWidth;
	}

	public int getSaveHeight() {
		return saveHeight;
	}

	public int getSaveX() {
		return x;
	}

	public int getSaveY() {
		return y;
	}

	public void saveURL(String text) {
		String filename = getClass().getResource(urlfile).getFile();
		File file = new File(filename);

		PrintWriter writer = null;
		try {
			writer = new PrintWriter(new BufferedWriter(new FileWriter(file)));
		} catch (IOException e) {
			e.printStackTrace();
		}
		writer.println(text);
		writer.close();
	}

	public String getURL() {
		setSquare();
		StringBuffer url = new StringBuffer();
		/*
		 * try { InputStream in = this.getClass().getResourceAsStream(urlfile);
		 * InputStreamReader stream = new InputStreamReader(in); BufferedReader
		 * reader = new BufferedReader(stream); url.append(reader.readLine()); }
		 * catch (Exception e) { e.printStackTrace(); }
		 */
		url.append(getUrlFromFile());
		url.append("?size=" + saveWidth + "x" + saveHeight);
		return url.append(layerList.getURL(x, y, saveWidth, saveHeight))
				.toString();
	}

	public String getUrlFromFile() {
		try {
			InputStream in = this.getClass().getResourceAsStream(urlfile);
			InputStreamReader stream = new InputStreamReader(in);
			BufferedReader reader = new BufferedReader(stream);
			return reader.readLine();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "";
	}

	public int[] getRectangle() {
		setSquare();
		int width = getSaveWidth() * BambooView.sizeOfOriginal + 1; // 摜̕
		int height = getSaveHeight() * BambooView.sizeOfOriginal + 1;
		return new int[] { width, height };
	}
}
