package ui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import controller.BambooController;

import model.*;

/**
 * BlueBamboo̕\SNX
 * 
 * @author teru
 * 
 */
public abstract class BambooView {
	public static final int sizeOfOriginal = 24;

	public static final int indent = 12; // ̗]

	private int cellsHorizontal = 20;
	private int cellsVertical = 20;

	private int sizeOfCell = 24;

	private JFrame frame;

	private BambooPanel bambooPanel;

	private CoordinateLabel coordinateLabel; // W\A傫\肷

	private JLabel sizeLabel; // ݂̃}XƔՖʂ̃TCY\

	private BambooController controller;

	private BambooModel model;

	private CommonButtonPanel commonButtonPanel;

	public BambooView(final BambooController controller, final BambooModel model) {
		this.controller = controller;
		this.model = model;
	}

	public int getPanelWidth() {
		return cellsHorizontal * sizeOfCell + 1;
	}

	public int getPanelHeight() {
		return cellsVertical * sizeOfCell + 1;
	}

	public int getCellsWidth() {
		return cellsHorizontal;
	}

	public int getCellsHeight() {
		return cellsVertical;
	}

	public int getSizeOfCell() {
		return sizeOfCell;
	}

	protected JLabel sizeLabel() {
		return sizeLabel;
	}

	protected JFrame getFrame() {
		return frame;
	}

	protected void disposeFrame() {
		frame.dispose();
	}

	public void setParameter(final int masuWidth, final int masuHeight,
			final int size) {
		cellsHorizontal = masuWidth;
		cellsVertical = masuHeight;
		sizeOfCell = size;
	}

	/**
	 * GUIׂ͂̕Ăō BambooControllerŌĂяo
	 * 
	 */
	public void createView() {
		// t[
		frame = new JFrame("BlueBamboo");
		Container c = frame.getContentPane();
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				processKey(e);
			}
		});

		// ̃pl
		bambooPanel = new BambooPanel(controller);
		controller.addCoodinateListener(bambooPanel);
		model.addLayerListener(bambooPanel);
		bambooPanel.setPreferredSize(new Dimension(controller.getPanelWidth(),
				controller.getPanelHeight()));
		c.add(bambooPanel, BorderLayout.CENTER);

		// TCYx̐ݒ
		sizeLabel = new JLabel(cellsHorizontal + "x" + cellsVertical
				+ ", Size:" + getSizeOfCell());
		sizeLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		commonButtonPanel = new CommonButtonPanel(controller, model);

		// ̕
		c.add(upperPanel(), BorderLayout.NORTH);

		// t[̉̕\
		StatusBar statusBar = new StatusBar(); // \pl
		statusBar.setLayout(new BorderLayout());
		coordinateLabel = new CoordinateLabel("(5,5)"); // x
		controller.addCoodinateListener(coordinateLabel);
		coordinateLabel.setHorizontalAlignment(SwingConstants.CENTER);
		statusBar.add(coordinateLabel, BorderLayout.CENTER);
		statusBar.add(commonButtonPanel, BorderLayout.EAST); // C[Ɖ]E]
		c.add(statusBar, BorderLayout.SOUTH);

		// t[Ŝ̒
		frame.pack();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		frame.setLocation((screenSize.width - frame.getWidth()) / 2,
				(screenSize.height - frame.getHeight()) / 2); // \ʒu𒲐
		frame.setResizable(false);
		frame.setVisible(true);
		frame.setFocusable(true);
		frame.requestFocus();
	}

	abstract protected JPanel upperPanel();

	protected CommonButtonPanel getCommonButtonPanel() {
		return commonButtonPanel;
	}

	protected JButton reverseButton() {
		return commonButtonPanel.getReverseButton();
	}
	
	protected JButton rotateButton() {
		return commonButtonPanel.getRotateButton();
	}
	
	public void showURL(String url) {
		final JDialog dialog = new JDialog(frame, "URL", true);
		JTextField urlField = new JTextField(url);
		urlField.setEditable(false);
		urlField.selectAll();
		urlField.copy();
		dialog.add(urlField, BorderLayout.CENTER);

		JPanel panel = new JPanel();
		JButton button = new JButton("OK");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialog.dispose();
			}
		});
		panel.add(button);
		dialog.add(panel, BorderLayout.SOUTH);
		dialog.pack();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		dialog.setLocation((screenSize.width - dialog.getWidth()) / 2,
				(screenSize.height - dialog.getHeight()) / 2); // \ʒu𒲐
		dialog.setVisible(true);
	}

	protected void processKey(KeyEvent e) {
		int keyCode = e.getKeyCode();

		if (keyCode == KeyEvent.VK_ESCAPE || keyCode == KeyEvent.VK_END
				|| (keyCode == KeyEvent.VK_C && e.isControlDown())) {
			frame.dispose();
		}

		switch (keyCode) {
		case KeyEvent.VK_UP:
			if (e.isShiftDown()) {
				controller.cursorShiftUp();
			} else {
				controller.cursorUp();
			}
			break;
		case KeyEvent.VK_DOWN:
			if (e.isShiftDown()) {
				controller.cursorShiftDown();
			} else {
				controller.cursorDown();
			}
			break;
		case KeyEvent.VK_RIGHT:
			if (e.isShiftDown()) {
				controller.cursorShiftRight();
			} else {
				controller.cursorRight();
			}
			break;
		case KeyEvent.VK_LEFT:
			if (e.isShiftDown()) {
				controller.cursorShiftLeft();
			} else {
				controller.cursorLeft();
			}
			break;
		case KeyEvent.VK_ENTER:
			controller.cursorEnter();
			break;
		case KeyEvent.VK_BACK_SPACE:
		case KeyEvent.VK_DELETE:
			controller.cursorDelete();
			break;
		case KeyEvent.VK_SPACE:
			controller.cursorSpace();
			break;
		case KeyEvent.VK_1:
			controller.setData(1);
			break;
		case KeyEvent.VK_2:
			controller.setData(2);
			break;
		case KeyEvent.VK_3:
			controller.setData(3);
			break;
		case KeyEvent.VK_4:
			controller.setData(4);
			break;
		case KeyEvent.VK_5:
			controller.setData(5);
			break;
		case KeyEvent.VK_6:
			controller.setData(6);
			break;
		case KeyEvent.VK_7:
			controller.setData(7);
			break;
		case KeyEvent.VK_8:
			controller.setData(8);
			break;
		case KeyEvent.VK_9:
			controller.setData(9);
			break;
		case KeyEvent.VK_0:
			controller.setData(0);
			break;
		case KeyEvent.VK_A:
			controller.setBamboo("left");
			break;
		case KeyEvent.VK_S:
			if (e.isControlDown()) {
				/* Ctrl-S ŃZ[u */
				// controller.save();
			} else {
				controller.setBamboo("horizontal");
			}
			break;
		case KeyEvent.VK_D:
			if (e.isControlDown()) {
				// C-d łׂẴV}
				controller.deleteAllIslands();
			} else {
				controller.setBamboo("right");
			}
			break;
		case KeyEvent.VK_T:
			controller.setBamboo("up");
			break;
		case KeyEvent.VK_G:
			controller.setBamboo("vertical");
			break;
		case KeyEvent.VK_B:
			controller.setBamboo("down");
			break;
		case KeyEvent.VK_F:
			controller.makeIsland();
			break;
		case KeyEvent.VK_X:
			controller.setCross();
			break;
		default:
			break;
		}
	}

	/**
	 * ݂t@Cɏ㏑邩ǂ
	 * 
	 * @param newFile
	 * @return ㏑OKȂtrue
	 */
	public boolean canReplaceFile(File newFile) {
		if (newFile.exists()) {
			int i = JOptionPane.showConfirmDialog(null, "㏑܂?",
					"choose one", JOptionPane.YES_NO_OPTION);
			if (i == JOptionPane.CLOSED_OPTION || i == JOptionPane.NO_OPTION) {
				return false;
			}
		}
		return true;
	}

	public void write(Graphics g, int left, int right, int top, int bottom) {
		bambooPanel.write(g, left, right, top, bottom);
	}

	public void eraseView() {
		frame.dispose();
	}

	public void enableLeftButton() {
		commonButtonPanel.enableLeftButton();
	}

	public void disableLeftButton() {
		commonButtonPanel.disableLeftButton();
	}

}
