package zuozuomu.watermark;

import junit.framework.*;

public class JikkenImageTest extends TestCase {

	public void testSegmentation() {
		JikkenImage image = new JikkenImage(100, 100);
		image.setPixelSegmentationValue(10, 20, 30);
		assertEquals(0, image.getPixelGrayValue(10, 20));
		image.setPixelSegmentationValue(10, 20, 167);
		assertEquals(255, image.getPixelGrayValue(10, 20));
	}

	public void testSetSegmentation() {
		JikkenImage image = new JikkenImage("test.jpg");
		image.setSegmentation();
		for (int i = 0; i < image.getWidth(); i++) {
			for (int j = 0; j < image.getHeight(); j++) {
				assertEquals(image.getPixelSegmentationValue(i, j), image
						.getPixelGrayValue(i, j));
			}
		}
	}

	public void testWriteSegmentation() {
		JikkenImage image = new JikkenImage("test.jpg");
		image.setSegmentation();
		image.embedIntData(0, 0, (byte) 10);
		image.writeJpegFile("segment.jpg","jpg");

		JikkenImage segment = new JikkenImage("segment.jpg");
		assertEquals(10, segment.extractIntData(0, 0));
	}

	private JikkenImage image = new JikkenImage("test.jpg");

	public void testConvertByteToInt() {
		byte[] data = { 5, 2, -8 };
		image.setSegmentation();
		image.embedIntArray(30, 28, data);
		image.writeJpegFile("array.jpg","jpg");

		JikkenImage segment = new JikkenImage("array.jpg");
		for (int i = 0; i < 3; i++)
			assertEquals(data[i], segment.extractIntArray(30, 28, 3)[i]);
	}
/*
	public void testArray() {
		byte[] data = { 0, 0, 5, 0, -1, 0 };
		image.setSegmentation();
		image.embedIntArray(50, 60, data, 2, 3);
		image.writeJpegFile("array2.jpg","jpg");

		JikkenImage array2 = new JikkenImage("array2.jpg");
		for (int i = 0; i < 3; i++)
			assertEquals(data[i + 2], array2.extractIntArray(50, 60, 3)[i]);
	}
*/
	public void testSquareArray() {
		byte[][] data = { { 1, 2, 3 }, { 4, 5, 6 } };
		image.setSegmentation();
		image.embedIntSquareArray(58, 28, data);
		image.writeJpegFile("sarray.jpg","jpg");

		JikkenImage sarray = new JikkenImage("sarray.jpg");
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 3; j++) {
				assertEquals(data[i][j], sarray.extractIntSquareArray(58, 28,
						2, 3)[i][j]);
			}
		}
	}
/*
	public void testSquareArray2() {
		byte[][] data = { { 0, 1, 0, 0 }, { -1, 20, 3, 2 }, { 39, 23, 4, 1 } };
		image.setSegmentation();
		image.embedIntSquareArray(20, 30, data, 1, 2, 2, 2);
		image.writeJpegFile("sarray2","jpg");

		JikkenImage sarray2 = new JikkenImage("sarray2.jpg");
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				assertEquals(data[i + 1][j + 2], sarray2.extractIntSquareArray(
						20, 30, 2, 2)[i][j]);
			}
		}
	}
	*/
}
